@@ -148,15 +148,29 @@ def prime_overlap(x, y):
     return sorted(overlap)
 
 
-def rescale(data, new_min=0, new_max=1, axis=None):
+def rescale(data, output_min=0, output_max=1, axis=None):
     """Rescales data to have range [new_min, new_max] along axis or axes indicated."""
     data = np.asarray(data)
     if np.issubdtype(data.dtype, np.integer):
         data = data.astype(float)
-    data_range = np.max(data, axis=axis, keepdims=True) - np.min(data, axis=axis, keepdims=True)
-    x = np.divide(data - np.min(data, axis=axis, keepdims=True), data_range, where=data_range > 0, out=np.zeros_like(data))
-    new_range = new_max - new_min
-    return (x * new_range) + new_min
+    max_val = np.max(data, axis=axis)
+    min_val = np.min(data, axis=axis)
+    data_range = max_val - min_val
+    if axis is not None:
+        min_val = np.expand_dims(min_val, axis)
+        data_range = np.expand_dims(data_range, axis)
+    x = np.divide(data - min_val, data_range, where=data_range > 0, out=np.zeros_like(data))
+    new_range = output_max - output_min
+    return (x * new_range) + output_min
+
+
+def order_normalization(data, order=2, axis=None):
+    norm = np.linalg.norm(data, order, axis)
+    norm = np.atleast_1d(norm)
+    norm[norm == 0] = 1
+    if axis is None:
+        return data / norm
+    return data / np.expand_dims(norm, axis)
 
 
 def clip(data, output_min=0, output_max=1, input_min=0, input_max=255):
@@ -177,11 +191,23 @@ def clip(data, output_min=0, output_max=1, input_min=0, input_max=255):
     """
     # TODO - Add tests for this
     data = np.clip(data, input_min, input_max)
-    scaler = (output_max - output_min) / (input_max - input_min)
-    bias = (input_min * output_min) / (input_max - input_min) - (input_min * output_max) / (input_max - input_min) + output_min
+    input_range = input_max - input_min
+    output_range = output_max - output_min
+    if input_range == 0:
+        return np.zeros_like(data) + output_min
+    scaler = output_range / input_range
+    bias = -input_min * scaler + output_min
     return data * scaler + bias
 
 
+def percentile_rescale(x, low_percentile=0.5, high_percentile=None, output_min=0, output_max=1):
+    if high_percentile is None:
+        high_percentile = 100 - low_percentile
+    low_percentile, high_percentile = sorted([low_percentile, high_percentile])
+    low_val, high_val = np.percentile(x, (low_percentile, high_percentile))
+    return clip(x, output_min, output_max, low_val, high_val)
+
+
 def percentile_normalize(x, low_percentile=0.5, high_percentile=None):
     """Normalize data after clipping to a percentile value
 
@@ -203,7 +229,13 @@ def percentile_normalize(x, low_percentile=0.5, high_percentile=None):
     low_percentile, high_percentile = sorted([low_percentile, high_percentile])
     low_val, high_val = np.percentile(x, (low_percentile, high_percentile))
     x = np.clip(x, low_val, high_val)
-    return (x - np.mean(x)) / np.std(x)
+    std_vals = np.std(x)
+    return np.divide(x - np.mean(x), std_vals, where=std_vals > 0, out=np.zeros_like(x))
+
+
+def relu(data):
+    """Return the rectified linear - max(data, 0)"""
+    return np.maximum(data, 0)
 
 
 def sigmoid(x, epsilon=1e-7):
@@ -469,3 +501,41 @@ def center_of_mass(input_arr):
 
     center_of_mass = np.sum(flat_mass * coords, axis=0) / total_mass
     return center_of_mass
+
+
+def max_signal(data, axis=None):
+    """Return the signed value with the largest absolute value along the given axis
+
+    Parameters
+    ----------
+    data : array_like
+        Input array
+    axis : int, optional
+        The axis to check across, otherwise the result uses the flattened array
+
+    NOTE
+    ----
+    If two values have the same absolute value and sign, the first one encountered along the given axis will be returned. If two values have the same absolute value and different signs, the positive value will be returned.
+    """
+    maxes = np.max(data, axis=axis)
+    mins = np.min(data, axis=axis)
+    return np.where(np.abs(mins) > maxes, mins, maxes)
+
+
+def argmax_signal(data, axis=None):
+    """Return the index of the signed value with the largest absolute value along an axis
+
+    Parameters
+    ----------
+    data : array_like
+        Input array
+    axis : int, optional
+        The axis to check across, otherwise the result uses the flattened array
+
+    NOTE
+    ----
+    If two values have the same absolute value and sign, the first one encountered along the given axis will be returned. If two values have the same absolute value and different signs, the positive value will be returned.
+    """
+    max_idx = np.argmax(data, axis=axis)
+    min_idx = np.argmin(data, axis=axis)
+    return np.where(np.abs(data.flat[min_idx]) > data.flat[max_idx], min_idx, max_idx)
